@@ -2,10 +2,20 @@ package Module::Build::Version;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = 0.82;
+$VERSION = 0.86;
 
 eval "use version $VERSION";
 if ($@) { # can't locate version files, use our own
+    # We might be here because version.pm was present but there was
+    # an error, so make we want to be sure users see what's happening
+    # We also need to clear %INC entries, as the "undef" for a broken
+    # version.pm can't be directly overwritten later.
+    if ( exists $INC{'version.pm'} && ! defined $INC{'version.pm'} ) {
+        warn "Error loading version.pm: $@\n";
+        warn "Using Module::Build::Version bundled version code instead.\n";
+        delete $INC{'version.pm'};
+        delete $INC{'version/vpp.pm'};
+    }
 
     # Avoid redefined warnings if an old version.pm was available
     delete $version::{$_} foreach keys %version::;
@@ -40,28 +50,148 @@ use vars qw(@ISA);
 
 1;
 __DATA__
-# stub version module to make everything else happy
+# XXX replace everything from here to the next XXX with the
+# contents of version.pm from CPAN and then edit to ensure
+# the XS clause is not invoked and comment out the line about
+# C<< eval "use version::vpp $VERSION"; >> since we provide
+# our own copy
+
 package version;
 
 use 5.005_04;
 use strict;
 
-use vars qw(@ISA $VERSION $CLASS *declare *qv);
+use vars qw(@ISA $VERSION $CLASS $STRICT $LAX *declare *qv);
 
-$VERSION = 0.82;
+$VERSION = 0.86;
 
 $CLASS = 'version';
 
-push @ISA, "version::vpp";
-local $^W;
-*version::qv = \&version::vpp::qv;
-*version::declare = \&version::vpp::declare;
-*version::_VERSION = \&version::vpp::_VERSION;
-if ($] > 5.009001 && $] <= 5.010000) {
-    no strict 'refs';
-    *{'version::stringify'} = \*version::vpp::stringify;
-    *{'version::(""'} = \*version::vpp::stringify;
-    *{'version::new'} = \*version::vpp::new;
+#--------------------------------------------------------------------------#
+# Version regexp components
+#--------------------------------------------------------------------------#
+
+# Fraction part of a decimal version number.  This is a common part of
+# both strict and lax decimal versions
+
+my $FRACTION_PART = qr/\.[0-9]+/;
+
+# First part of either decimal or dotted-decimal strict version number.
+# Unsigned integer with no leading zeroes (except for zero itself) to
+# avoid confusion with octal.
+
+my $STRICT_INTEGER_PART = qr/0|[1-9][0-9]*/;
+
+# First part of either decimal or dotted-decimal lax version number.
+# Unsigned integer, but allowing leading zeros.  Always interpreted
+# as decimal.  However, some forms of the resulting syntax give odd
+# results if used as ordinary Perl expressions, due to how perl treats
+# octals.  E.g.
+#   version->new("010" ) == 10
+#   version->new( 010  ) == 8
+#   version->new( 010.2) == 82  # "8" . "2"
+
+my $LAX_INTEGER_PART = qr/[0-9]+/;
+
+# Second and subsequent part of a strict dotted-decimal version number.
+# Leading zeroes are permitted, and the number is always decimal.
+# Limited to three digits to avoid overflow when converting to decimal
+# form and also avoid problematic style with excessive leading zeroes.
+
+my $STRICT_DOTTED_DECIMAL_PART = qr/\.[0-9]{1,3}/;
+
+# Second and subsequent part of a lax dotted-decimal version number.
+# Leading zeroes are permitted, and the number is always decimal.  No
+# limit on the numerical value or number of digits, so there is the
+# possibility of overflow when converting to decimal form.
+
+my $LAX_DOTTED_DECIMAL_PART = qr/\.[0-9]+/;
+
+# Alpha suffix part of lax version number syntax.  Acts like a
+# dotted-decimal part.
+
+my $LAX_ALPHA_PART = qr/_[0-9]+/;
+
+#--------------------------------------------------------------------------#
+# Strict version regexp definitions
+#--------------------------------------------------------------------------#
+
+# Strict decimal version number.
+
+my $STRICT_DECIMAL_VERSION =
+    qr/ $STRICT_INTEGER_PART $FRACTION_PART? /x;
+
+# Strict dotted-decimal version number.  Must have both leading "v" and
+# at least three parts, to avoid confusion with decimal syntax.
+
+my $STRICT_DOTTED_DECIMAL_VERSION =
+    qr/ v $STRICT_INTEGER_PART $STRICT_DOTTED_DECIMAL_PART{2,} /x;
+
+# Complete strict version number syntax -- should generally be used
+# anchored: qr/ \A $STRICT \z /x
+
+$STRICT =
+    qr/ $STRICT_DECIMAL_VERSION | $STRICT_DOTTED_DECIMAL_VERSION /x;
+
+#--------------------------------------------------------------------------#
+# Lax version regexp definitions
+#--------------------------------------------------------------------------#
+
+# Lax decimal version number.  Just like the strict one except for
+# allowing an alpha suffix or allowing a leading or trailing
+# decimal-point
+
+my $LAX_DECIMAL_VERSION =
+    qr/ $LAX_INTEGER_PART (?: \. | $FRACTION_PART $LAX_ALPHA_PART? )?
+	|
+	$FRACTION_PART $LAX_ALPHA_PART?
+    /x;
+
+# Lax dotted-decimal version number.  Distinguished by having either
+# leading "v" or at least three non-alpha parts.  Alpha part is only
+# permitted if there are at least two non-alpha parts. Strangely
+# enough, without the leading "v", Perl takes .1.2 to mean v0.1.2,
+# so when there is no "v", the leading part is optional
+
+my $LAX_DOTTED_DECIMAL_VERSION =
+    qr/
+	v $LAX_INTEGER_PART (?: $LAX_DOTTED_DECIMAL_PART+ $LAX_ALPHA_PART? )?
+	|
+	$LAX_INTEGER_PART? $LAX_DOTTED_DECIMAL_PART{2,} $LAX_ALPHA_PART?
+    /x;
+
+# Complete lax version number syntax -- should generally be used
+# anchored: qr/ \A $LAX \z /x
+#
+# The string 'undef' is a special case to make for easier handling
+# of return values from ExtUtils::MM->parse_version
+
+$LAX =
+    qr/ undef | $LAX_DECIMAL_VERSION | $LAX_DOTTED_DECIMAL_VERSION /x;
+
+#--------------------------------------------------------------------------#
+
+# XXX NO XS FOR US
+# eval "use version::vxs $VERSION";
+if ( 1 ) { # XXX force pure perl
+#   XXX We provide our own vpp
+#    eval "use version::vpp $VERSION"; # don't tempt fate
+#    die "$@" if ( $@ );
+    push @ISA, "version::vpp";
+    local $^W;
+    *version::qv = \&version::vpp::qv;
+    *version::declare = \&version::vpp::declare;
+    *version::_VERSION = \&version::vpp::_VERSION;
+    if ($] >= 5.009000 && $] < 5.011004) {
+	no strict 'refs';
+	*version::stringify = \&version::vpp::stringify;
+	*{'version::(""'} = \&version::vpp::stringify;
+	*version::new = \&version::vpp::new;
+	*version::parse = \&version::vpp::parse;
+    }
+}
+else {
+  # XXX NO XS FOR US -- deleted it
 }
 
 # Preloaded methods go here.
@@ -81,7 +211,7 @@ sub import {
 	map { $args{$_} = 1 } @_
     }
     else { # no parameters at all on use line
-    	%args =
+    	%args = 
 	(
 	    qv => 1,
 	    'UNIVERSAL::VERSION' => 1,
@@ -89,32 +219,47 @@ sub import {
     }
 
     my $callpkg = caller();
-
+    
     if (exists($args{declare})) {
-	*{$callpkg."::declare"} =
+	*{$callpkg.'::declare'} = 
 	    sub {return $class->declare(shift) }
 	  unless defined(&{$callpkg.'::declare'});
     }
 
     if (exists($args{qv})) {
-	*{$callpkg."::qv"} =
+	*{$callpkg.'::qv'} =
 	    sub {return $class->qv(shift) }
-	  unless defined(&{"$callpkg\::qv"});
+	  unless defined(&{$callpkg.'::qv'});
     }
 
     if (exists($args{'UNIVERSAL::VERSION'})) {
 	local $^W;
-	*UNIVERSAL::VERSION = \&version::_VERSION;
+	*UNIVERSAL::VERSION 
+		= \&version::_VERSION;
     }
 
     if (exists($args{'VERSION'})) {
-	*{$callpkg."::VERSION"} = \&version::_VERSION;
+	*{$callpkg.'::VERSION'} = \&version::_VERSION;
+    }
+
+    if (exists($args{'is_strict'})) {
+	*{$callpkg.'::is_strict'} = \&version::is_strict
+	  unless defined(&{$callpkg.'::is_strict'});
+    }
+
+    if (exists($args{'is_lax'})) {
+	*{$callpkg.'::is_lax'} = \&version::is_lax
+	  unless defined(&{$callpkg.'::is_lax'});
     }
 }
 
+sub is_strict	{ defined $_[0] && $_[0] =~ qr/ \A $STRICT \z /x }
+sub is_lax	{ defined $_[0] && $_[0] =~ qr/ \A $LAX \z /x }
+
 1;
 
-# replace everything from here to the end with the current version/vpp.pm
+# XXX replace everything from here to the end with the current version/vpp.pm
+
 package charstar;
 # a little helper class to emulate C char* semantics in Perl
 # so that prescan_version can use the same code as in C
@@ -238,7 +383,7 @@ use strict;
 use POSIX qw/locale_h/;
 use locale;
 use vars qw ($VERSION @ISA @REGEXS);
-$VERSION = 0.82;
+$VERSION = 0.86;
 
 use overload (
     '""'       => \&stringify,
@@ -936,25 +1081,57 @@ sub _verify {
     }
 }
 
+sub _is_non_alphanumeric {
+    my $s = shift;
+    $s = new charstar $s;
+    while ($s) {
+	return 0 if isSPACE($s); # early out
+	return 1 unless (isALPHA($s) || isDIGIT($s) || $s =~ /[.-]/);
+	$s++;
+    }
+    return 0;
+}
+
 sub _un_vstring {
     my $value = shift;
     # may be a v-string
-    if ( $] >= 5.006_000 && length($value) >= 3 && $value !~ /[._]/) {
-	foreach my $char (split(//,$value)) {
-	    # if one of the characters is non-text assume v-string
-	    if (ord($char) < ord(" ")) {
-		my $tvalue = sprintf("v%vd",$value);
-		if ( $tvalue =~ /^v\d+(\.\d+){2,}$/ ) {
-		    # must be a v-string
-		    $value = $tvalue;
-		}
-		last;
+    if ( length($value) >= 3 && $value !~ /[._]/ 
+	&& _is_non_alphanumeric($value)) {
+	my $tvalue;
+	if ( $] ge 5.008_001 ) {
+	    $tvalue = _find_magic_vstring($value);
+	    $value = $tvalue if length $tvalue;
+	}
+	elsif ( $] ge 5.006_000 ) {
+	    $tvalue = sprintf("v%vd",$value);
+	    if ( $tvalue =~ /^v\d+(\.\d+){2,}$/ ) {
+		# must be a v-string
+		$value = $tvalue;
 	    }
 	}
     }
     return $value;
 }
 
+sub _find_magic_vstring {
+    my $value = shift;
+    my $tvalue = '';
+    require B;
+    my $sv = B::svref_2object(\$value);
+    my $magic = ref($sv) eq 'B::PVMG' ? $sv->MAGIC : undef;
+    while ( $magic ) {
+	if ( $magic->TYPE eq 'V' ) {
+	    $tvalue = $magic->PTR;
+	    $tvalue =~ s/^v?(.+)$/v$1/;
+	    last;
+	}
+	else {
+	    $magic = $magic->MOREMAGIC;
+	}
+    }
+    return $tvalue;
+}
+
 sub _VERSION {
     my ($obj, $req) = @_;
     my $class = ref($obj) || $obj;
